@@ -35,13 +35,16 @@
 #include <unistd.h>
 #include <errno.h>
 #include <zck.h>
+#ifdef _WIN32
+#include <fcntl.h>
+#endif
 
 #include "zck_private.h"
 
 
 
 /* If lead format changes, this needs to be changed */
-int PUBLIC zck_get_min_download_size() {
+int ZCK_PUBLIC_API zck_get_min_download_size() {
     /* Magic + hash type + hash digest + header size */
     return 5 + MAX_COMP_SIZE*2 + get_max_hash_size();
 }
@@ -97,11 +100,16 @@ static int hex_to_int (char c) {
     return result;
 }
 
+
 static char *ascii_checksum_to_bin (zckCtx *zck, char *checksum,
                                     int checksum_length) {
     char *raw_checksum = zmalloc(checksum_length/2);
     char *rp = raw_checksum;
     int buf = 0;
+    if (!raw_checksum) {
+       zck_log(ZCK_LOG_ERROR, "OOM in %s", __func__);
+       return NULL;
+    }
     for (int i=0; i<checksum_length; i++) {
         // Get integer value of hex checksum character.  If -1 is returned, then
         // the character wasn't actually hex, so return NULL
@@ -122,13 +130,17 @@ static char *ascii_checksum_to_bin (zckCtx *zck, char *checksum,
 
 void *zmalloc(size_t size) {
     void *ret = calloc(1, size);
-    assert(ret);
     return ret;
 }
 
 void *zrealloc(void *ptr, size_t size) {
     void *ret = realloc(ptr, size);
-    assert(ret);
+    /*
+     * Realloc does not touch the original block if fails.
+     * Policy is to free memory and returns with error (Null)
+     */
+    if (!ret && ptr)
+        free(ptr);
     return ret;
 }
 
@@ -138,7 +150,12 @@ int get_tmp_fd(zckCtx *zck) {
     int temp_fd;
     char *fname = NULL;
     char template[] = "zcktempXXXXXX";
+
+    #ifdef _WIN32
+    char *tmpdir = getenv("TEMP");
+    #else
     char *tmpdir = getenv("TMPDIR");
+    #endif
 
     if(tmpdir == NULL) {
         tmpdir = "/tmp/";
@@ -148,31 +165,52 @@ int get_tmp_fd(zckCtx *zck) {
     }
 
     fname = zmalloc(strlen(template) + strlen(tmpdir) + 2);
+    if (!fname) {
+       zck_log(ZCK_LOG_ERROR, "OOM in %s", __func__);
+       return -ENOMEM;
+    }
     int i=0;
     for(i=0; i<strlen(tmpdir); i++)
         fname[i] = tmpdir[i];
     int offset = i;
+    #ifdef _WIN32
+    fname[offset] = '\\';
+    #else
     fname[offset] = '/';
+    #endif
     offset++;
     for(i=0; i<strlen(template); i++)
         fname[offset + i] = template[i];
     offset += i;
     fname[offset] = '\0';
 
+    typedef int mode_t;
     mode_t old_mode_mask;
+
+    #ifdef _WIN32
+    errno_t out = _mktemp_s(
+        fname,
+        offset + 1
+    );
+    temp_fd = open(fname, O_CREAT | O_EXCL | O_RDWR | O_BINARY);
+    #else
     old_mode_mask = umask (S_IXUSR | S_IRWXG | S_IRWXO);
     temp_fd = mkstemp(fname);
     umask(old_mode_mask);
+    #endif
     if(temp_fd < 0) {
         free(fname);
         set_error(zck, "Unable to create temporary file");
         return -1;
     }
+#ifndef _WIN32
+    // Files with open file handle cannot be removed on Windows
     if(unlink(fname) < 0) {
         free(fname);
         set_error(zck, "Unable to delete temporary file");
         return -1;
     }
+#endif
     free(fname);
     return temp_fd;
 }
@@ -188,6 +226,10 @@ bool import_dict(zckCtx *zck) {
 
     zck_log(ZCK_LOG_DEBUG, "Reading compression dict");
     char *data = zmalloc(size);
+    if (!data) {
+       zck_log(ZCK_LOG_ERROR, "OOM in %s", __func__);
+       return false;
+    }
     if(comp_read(zck, data, size, 0) != size) {
         set_error(zck, "Error reading compressed dict");
         return false;
@@ -204,10 +246,14 @@ bool import_dict(zckCtx *zck) {
     return true;
 }
 
-bool PUBLIC zck_set_soption(zckCtx *zck, zck_soption option, const char *value,
+bool ZCK_PUBLIC_API zck_set_soption(zckCtx *zck, zck_soption option, const char *value,
                             size_t length) {
     VALIDATE_BOOL(zck);
     char *data = zmalloc(length);
+    if (!data) {
+       zck_log(ZCK_LOG_ERROR, "OOM in %s", __func__);
+       return false;
+    }
     memcpy(data, value, length);
 
     /* Validation options */
@@ -226,14 +272,17 @@ bool PUBLIC zck_set_soption(zckCtx *zck, zck_soption option, const char *value,
         }
         if(chk_type.digest_size*2 != length) {
             free(data);
-            set_fatal_error(zck, "Hash digest size mismatch for header "
-                                 "validation\n"
-                                 "Expected: %i\nProvided: %lu",
-                                 chk_type.digest_size*2, length);
+            set_fatal_error(
+                zck,
+                "Hash digest size mismatch for header validation\n"
+                "Expected: %i\nProvided: %llu",
+                chk_type.digest_size*2,
+                (long long unsigned) length
+            );
             return false;
         }
-        zck_log(ZCK_LOG_DEBUG, "Setting expected hash to (%s)%s",
-                zck_hash_name_from_type(zck->prep_hash_type), data);
+        zck_log(ZCK_LOG_DEBUG, "Setting expected hash to (%s)%.*s",
+                zck_hash_name_from_type(zck->prep_hash_type), length, data);
         zck->prep_digest = ascii_checksum_to_bin(zck, data, length);
         free(data);
         if(zck->prep_digest == NULL) {
@@ -255,7 +304,7 @@ bool PUBLIC zck_set_soption(zckCtx *zck, zck_soption option, const char *value,
     return true;
 }
 
-bool PUBLIC zck_set_ioption(zckCtx *zck, zck_ioption option, ssize_t value) {
+bool ZCK_PUBLIC_API zck_set_ioption(zckCtx *zck, zck_ioption option, ssize_t value) {
     VALIDATE_BOOL(zck);
 
     /* Set hash type */
@@ -270,8 +319,8 @@ bool PUBLIC zck_set_ioption(zckCtx *zck, zck_ioption option, ssize_t value) {
     } else if(option == ZCK_VAL_HEADER_HASH_TYPE) {
         VALIDATE_READ_BOOL(zck);
         if(value < 0) {
-            set_error(zck, "Header hash type can't be less than zero: %li",
-                      value);
+            set_error(zck, "Header hash type can't be less than zero: %lli",
+                      (long long) value);
             return false;
         }
         /* Make sure that header hash type is set before the header digest,
@@ -286,16 +335,29 @@ bool PUBLIC zck_set_ioption(zckCtx *zck, zck_ioption option, ssize_t value) {
         VALIDATE_READ_BOOL(zck);
         if(value < 0) {
             set_error(zck,
-                      "Header size validation can't be less than zero: %li",
-                      value);
+                      "Header size validation can't be less than zero: %lli",
+                      (long long) value);
             return false;
         }
         zck->prep_hdr_size = value;
 
+    } else if(option == ZCK_UNCOMP_HEADER) {
+        zck->has_uncompressed_source = 1;
+        /* Uncompressed source requires chunk checksums to be a minimum of SHA-256 */
+        if(zck->chunk_hash_type.type == ZCK_HASH_SHA1 ||
+           zck->chunk_hash_type.type == ZCK_HASH_SHA512_128) {
+            if(!set_chunk_hash_type(zck, ZCK_HASH_SHA256))
+                return false;
+        }
+
     /* Hash options */
     } else if(option < 100) {
         /* Currently no hash options other than setting hash type, so bail */
-        set_error(zck, "Unknown option %lu", value);
+        set_error(
+            zck,
+            "Unknown option %llu",
+            (long long unsigned) value
+        );
         return false;
 
     /* Compression options */
@@ -311,7 +373,7 @@ bool PUBLIC zck_set_ioption(zckCtx *zck, zck_ioption option, ssize_t value) {
     return true;
 }
 
-bool PUBLIC zck_close(zckCtx *zck) {
+bool ZCK_PUBLIC_API zck_close(zckCtx *zck) {
     VALIDATE_BOOL(zck);
 
     if(zck->mode == ZCK_MODE_WRITE) {
@@ -339,7 +401,7 @@ bool PUBLIC zck_close(zckCtx *zck) {
     return true;
 }
 
-void PUBLIC zck_free(zckCtx **zck) {
+void ZCK_PUBLIC_API zck_free(zckCtx **zck) {
     if(zck == NULL || *zck == NULL)
         return;
     zck_clear(*zck);
@@ -347,15 +409,19 @@ void PUBLIC zck_free(zckCtx **zck) {
     *zck = NULL;
 }
 
-zckCtx PUBLIC *zck_create() {
+zckCtx ZCK_PUBLIC_API *zck_create() {
     zckCtx *zck = zmalloc(sizeof(zckCtx));
+    if (!zck) {
+       zck_log(ZCK_LOG_ERROR, "OOM in %s", __func__);
+       return false;
+    }
     zck_clear_error(NULL);
     zck->prep_hash_type = -1;
     zck->prep_hdr_size = -1;
     return zck;
 }
 
-bool PUBLIC zck_init_adv_read (zckCtx *zck, int src_fd) {
+bool ZCK_PUBLIC_API zck_init_adv_read (zckCtx *zck, int src_fd) {
     VALIDATE_BOOL(zck);
 
     zck->mode = ZCK_MODE_READ;
@@ -363,7 +429,7 @@ bool PUBLIC zck_init_adv_read (zckCtx *zck, int src_fd) {
     return true;
 }
 
-bool PUBLIC zck_init_read (zckCtx *zck, int src_fd) {
+bool ZCK_PUBLIC_API zck_init_read (zckCtx *zck, int src_fd) {
     VALIDATE_BOOL(zck);
 
     if(!zck_init_adv_read(zck, src_fd)) {
@@ -384,7 +450,7 @@ bool PUBLIC zck_init_read (zckCtx *zck, int src_fd) {
     return true;
 }
 
-bool PUBLIC zck_init_write (zckCtx *zck, int dst_fd) {
+bool ZCK_PUBLIC_API zck_init_write (zckCtx *zck, int dst_fd) {
     VALIDATE_BOOL(zck);
 
     zck->mode = ZCK_MODE_WRITE;
@@ -409,12 +475,12 @@ bool PUBLIC zck_init_write (zckCtx *zck, int dst_fd) {
     return true;
 }
 
-int PUBLIC zck_get_fd(zckCtx *zck) {
+int ZCK_PUBLIC_API zck_get_fd(zckCtx *zck) {
     VALIDATE_BOOL(zck);
     return zck->fd;
 }
 
-bool PUBLIC zck_set_fd(zckCtx *zck, int fd) {
+bool ZCK_PUBLIC_API zck_set_fd(zckCtx *zck, int fd) {
     VALIDATE_BOOL(zck);
     zck->fd = fd;
     return true;
